// Sample code to fill out a matrix from STL vectors
// The code demonstrates techniques of filling in matrices in a regular
// and irregular way.
// The example is due to Elias Kyriakides.
//
// $Id: sample_fill.cc,v 1.2 2002/11/21 03:01:27 oleg Exp oleg $

#include "LAStreams.h"
#if defined(__GNUC__) || __GNUC__ >= 3
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <vector>
using std::vector;
#else
#include <iostream.h>
#include <vector>
#endif

// for VC 7.0 Markus Bader 17.April 2004
using namespace std ;

int main(void)
{
  cout << "\n\nFilling in matrices from STL vectors" << endl;

  vector<double> v1(5), v2(5);
  // Fill in the vectors with sample data
  v1[0] = 10; v1[1] = 11; v1[2] = 12; v1[3] = 13; v1[4] = 14; 
  v2[0] = 20; v2[1] = 21; v2[2] = 22; v2[3] = 23; v2[4] = 24; 

  {
    cout << "Example 1: filling in a matrix in a irregular way" << endl;

    Matrix A(20,5);  // The matrix is initialized to zero upon construction

    {
      MatrixDA Ad(A);  	// Construct a random-access view to fill in the matrix
			// Now fill in the matrix in an irregular way
      Ad(1,2) = v1[0];
      Ad(2,5) = v2[0];
      Ad(3,1) = v2[1];
      Ad(3,5) = v1[1];
      Ad(18,4)= 2*v1[2]+v2[2];
			// The Ad view will be destroyed when this scope ends
    }

    // Add a few values to the 19-th row of the matrix
    {
      MatrixRow Ar(A,19);
      Ar(1) = v1[2]; Ar(4) = v1[1] + v2[3];
    }

    A.print("Irregularly filled matrix");
  }

  {
    cout << "Example 2: filling in a matrix in a regular way" << endl;

    Matrix A(20,5);  // The matrix is initialized to zero upon construction

    {
      // We assign v1 to a part of the 2nd column of the matrix A:
      // A[i,2] = v1[i-1], i=1..5
    
      // Create a stream over the second column of Matrix A
      LAStreamOut As(MatrixColumn(A,2));
      
      // Fill in the stream from an STL vector

      for(vector<double>::const_iterator vi=v1.begin(); vi != v1.end(); vi++)
	As.get() = *vi;
    
      // Assign v2 to a part of the sub-diagonal of the matrix
      // A[7+j,j] = v2[j-1], j=1..5

      // Create a block stream over A[8..20,1..5]
      LABlockStreamOut Ab(A,IRange::from(8),IRange::from(1));
      for(vector<double>::const_iterator vi=v2.begin(); vi != v2.end(); vi++)
      {
	Ab.peek() = *vi;
	Ab.ignore(13+1); // 13 is the number of the rows in the slice
			 // Ignoring 13 elements moves the stream from
			 // A[7+j,j] to A[7+j+1,j+1]
      }
    }
    A.print("Regularly filled matrix");
  }

  cout << "\nDone" << endl;
}
   

